<#PSScriptInfo

.VERSION 3.0.1

.AUTHOR Norbert Meier

.COMPANYNAME LISEGA SE

.COPYRIGHT (c) 2025 LISEGA SE
#>

<#
.SYNOPSIS
This Powershell script installes the LISEGA Smart3D catalog and H&S Plug-IN V 3.0.1

.DESCRIPTION
You can use this script to perform the required steps to install LISEGA Symbol and Plug-In DLL's and the Catalog part images and to configure Smart3D

.EXAMPLE
./InstallLisegaV3.ps1 -S3DinstDir 'C:\Program Files (x86)\Smart3D' -SymbolShare '\\server23\S3DShare\' -DBserver 'ServerSql01' -DBcatalog 'S3dTrain' -DBcatalogSchema 'S3dTrainSchema' -ViewBulkloadLogs

.NOTES

.PARAMETER S3DinstDir
Installation directory of Smart3D

.PARAMETER SymbolShare
Symbol share folder for catalog database

.PARAMETER DBserver
SQL server Computer name for catalog database

.PARAMETER DBcatalog
Catalog database name where the bulkload should be run

.PARAMETER DBcatalogSchema
Schema database name for Catalog database

.PARAMETER DBserverSchema
SQL server Computer name for schema catalog database
If omitted, DBserver is used
	
.PARAMETER ViewBulkloadLogs
Show log file content of bulkload during execution

.PARAMETER SkipFileCopy
Skip copying DLL's and part images (Only for testing)

.PARAMETER SkipPrjMngmnt
Skip starting of Smart3D project management and and waiting for it to close (Only for testing)

.PARAMETER SkipBulkloads
Skip all bulkload runs (Only for testing)

#>
# This makes the PowerShell script to accept only named parameters!!
[CmdletBinding(PositionalBinding=$False)]
param(
	[string]$S3DinstDir, 
	[string]$SymbolShare,
	[string]$DBserver,
	[string]$DBcatalog,
	[string]$DBcatalogSchema,
	[string]$DBserverSchema="",
	[switch]$ViewBulkloadLogs=$False, 
	[switch]$SkipFileCopy=$False, 
	[switch]$SkipPrjMngmnt=$False,
	[switch]$SkipBulkloads=$False
);

# enable exception handling
$ErrorColorActionPreference = "Stop"

#requires -version 5.0


$MessageColor = @{}
$DebugColor   = @{ForegroundColor = "Gray"}
$InfoColor    = @{ForegroundColor = "Cyan"}
$WarningColor = @{ForegroundColor = "Yellow"}
$SuccessColor = @{ForegroundColor = "Green"}
$ErrorColor   = @{ForegroundColor = "Red"}

# This is the main program part. A script block is used, so that this can be written in front of all other function definitions and execute it in the last line of the script!
$Main = {
	write-host ""
	write-host @InfoColor "---------------------------------------------------"
	write-host @InfoColor "Installing LISEGA V 3.0.1 catalog and plug-in ..."
	write-host @InfoColor "---------------------------------------------------"

	try
	{
		CheckCmdLineParameter
	}
	catch
	{
		write-Host ""
		Write-Host @ErrorColor "CheckCmdLineParameter failed: $_"
		write-Host ""
		write-Host @WarningColor "Usage: InstallLisegaV3.bat -S3DinstDir <Smart3D installation directory>"
		write-Host @WarningColor "                           -SymbolShare <Symbol share>"
		write-Host @WarningColor "                           -DBserver <Database server name>"
		write-Host @WarningColor "                           -DBcatalog <Catalog database name>"
		write-Host @WarningColor "                           -DBcatalogSchema <Catalog schema database name>"
		write-Host @WarningColor "                           [-SchemaDBserver <Schema Database server name>]"
		write-Host @WarningColor "                           [-ViewBulkloadLogs]"
		write-Host ""
		write-Host ""
		exit 99
	}

	InitializeVariables 

	LogVariables

	# ************* Now start execution of the required steps and exit directly, if a step fails and do not perform any further installation! ***********
	# #### Step 0: Check input and depending folders and files ####
	try
	{
		CheckRequirements
	}
	catch
	{
		Write-Host @ErrorColor "CheckRequirements failed: $_"
		exit 1
	}

	# *** Copying files to their destination folders *****
	if ($SkipFileCopy)
	{
		write-host @InfoColor "Skipping DLL file copy, because switch SkipFileCopy was detected"
	}
	else
	{
		# #### Step 1: Copy Symbol DLL's ####
		try
		{
			CopyDlls2SymbolShare
		}
		catch
		{
			Write-Host @ErrorColor "CopyDlls2SymbolShare failed: $_"
			exit 2
		}
		# #### Step 2: Copy Part Images ####
		try
		{
			CopyPartImages
		}
		catch 
		{
			Write-Host @ErrorColor "CopyPartImages failed: $_"
			exit 3
		}
	}
	
	# #### Step 3: Start Project Management and wait until closed .... ####
	if ($SkipPrjMngmnt)
	{
		write-host @InfoColor "Skipping DLL file copy, because switch SkipPrjMngmnt was detected"
	}
	else
	{
		try
		{
			StartS3dPrjMngmntAndWait
		}
		catch 
		{
			Write-Host @ErrorColor "StartS3dPrjMngmntAndWait failed: $_"
			exit 3
		}
	}
	
	# #### Step 4: Bulkload all LISEGA Excel files ... ####
	if ($SkipBulkloads)
	{
		write-host @InfoColor "Skipping Bulkloads, because switch SkipBulkloads was detected"
	}
	else
	{
		try
		{
			BulkloadAllLisegaFiles
		}
		catch 
		{
			Write-Host @ErrorColor "BulkloadAllLisegaFiles failed: $_"
			exit 4
		}
	}
	
	if ($SkipFileCopy -and $SkipPrjMngmnt -and $SkipBulkloads)
	{ 
		write-host @InfoColor "No installation step performed, because all Skip switches were given!"
	}
	else
	{
		Write-Host @SuccessColor "-----------------------------------------------------------------------------------------"
		Write-Host @SuccessColor "|                                                                                       |"
		Write-Host @SuccessColor "|                                                                                       |"
		Write-Host @SuccessColor "|                                                                                       |"
		Write-Host @SuccessColor "|            LISEGA V 3.0.1 catalog and H&S plug-in installation successful!            |"
		Write-Host @SuccessColor "|                                                                                       |"
		Write-Host @SuccessColor "|                                                                                       |"
		Write-Host @SuccessColor "|                                                                                       |"
		Write-Host @SuccessColor "-----------------------------------------------------------------------------------------"
	}
	
}

function CheckCmdLineParameter
{
	write-host "Checking command line parameters ..."
	if ( [string]::IsNullOrEmpty($S3DinstDir))	{ throw "Parameter S3DinstDir not set!" }
	if ( [string]::IsNullOrEmpty($SymbolShare))	{ throw "Parameter SymbolShare not set!" }
	if ( [string]::IsNullOrEmpty($DBserver))	{ throw "Parameter DBserver not set!" }
	if ( [string]::IsNullOrEmpty($DBcatalog))	{ throw "Parameter DBcatalog not set!" }
	if ( [string]::IsNullOrEmpty($DBcatalogSchema))	{ throw "Parameter DBcatalogSchema not set!" }
}

function InitializeVariables 
{
	$script:PackageSource = $PSScriptRoot
	$script:DllSourceParh = [IO.Path]::Combine($PackageSource, "SymbolShare", "CustomSymbols")
	$script:PartImagesSourceParh= [IO.Path]::Combine($PackageSource, "PartImages")
	if ([string]::IsNullOrEmpty($script:DBserverSchema))
	{
		$script:DBserverSchema = $DBserver
		$script:DBserverSchemaWasEmpty = $true
	}
	$script:S3dInstDirCoreRuntime = [IO.Path]::Combine($S3dInstDir, "Core", "Runtime")
	$script:S3dInstDirCoreContainerBin = [IO.Path]::Combine($S3dInstDir, "Core", "Container", "Bin")
	$env:PATH += ";$($script:S3dInstDirCoreRuntime)"
	$env:PATH += ";$($script:S3dInstDirCoreContainerBin)"
	$script:S3dHostExe = [IO.Path]::Combine($script:S3dInstDirCoreContainerBin, "S3DHost.exe")
	$script:PrjEnvXml = [IO.Path]::Combine($S3dInstDir, "ProjectMgmt", "Environment", "Xml", "ProjectEnv.xml")
	$script:BulkloadExe = [IO.Path]::Combine($S3dInstDir, "CatalogData", "BulkLoad", "Bin", "Bulkload.exe")
	$script:SymbolShareCustomSymbols = [IO.Path]::Combine($SymbolShare, "Custom Symbols")
	$script:SymbolSharePartImages = [IO.Path]::Combine($SymbolShare, "Data", "HangersAndSupports", "PartImages")
}

function CheckRequirements
{
	write-host "Checking if source folders exists ..."
	if (!(Test-Path $DllSourceParh -PathType Container))				{ throw "Source Folder '$DllSourceParh' not found -- '$PackageSource' seems not to be a valid installation package folder" }
	if (!(Test-Path $PartImagesSourceParh -PathType Container))			{ throw "Source Folder '$PartImagesSourceParh' not found -- '$PackageSource' seems not to be a valid installation package folder" }
	
	write-host "Checking if destination folders exists ..."
	if (!(Test-Path $S3dInstDir -PathType Container))					{ throw "Smart3D Installation folder '$S3dInstDir' does not exist" }
	if (!(Test-Path $S3dInstDirCoreRuntime -PathType Container))		{ throw "Path '$S3dInstDirCoreRuntime' does not exist - '$S3dInstDir' seens not to be the  Smart3D Installation folder" }
	if (!(Test-Path $S3dInstDirCoreContainerBin -PathType Container))	{ throw "Path '$S3dInstDirCoreContainerBin' does not exist - '$S3dInstDir' seens not to be the  Smart3D Installation folder" }
	if (!(Test-Path $SymbolShare -PathType Container))					{ throw "Symbol share Folder '$SymbolShare' not found" }
	if (!(Test-Path $SymbolShareCustomSymbols -PathType Container))		{ throw "Folder 'SymbolShareCustomSymbols' not found -- '$SymbolShare' seems not to be the symbol share" }
	if (!(Test-Path $SymbolSharePartImages -PathType Container))		{ throw "Folder 'SymbolSharePartImages' not found -- '$SymbolShare' seems not to be the symbol share" }

	write-host "Checking if required programs and files exists ..."
	if (!(Test-Path $S3dHostExe -PathType Leaf))						{ throw "File '$S3dHostExe' not found - '$S3dInstDir' seens not to be the  Smart3D Installation folder" }
	if (!(Test-Path $PrjEnvXml -PathType Leaf)) 						{ throw "File '$PrjEnvXml' not found - '$S3dInstDir' seens not to be the  Smart3D Installation folder" }
	if (!(Test-Path $BulkloadExe -PathType Leaf)) 						{ throw "File '$BulkloadExe' not found - '$S3dInstDir' seens not to be the  Smart3D Installation folder" }
	
}

function CopyDlls2SymbolShare
{
	write-host @InfoColor "---------------------------------------------------------------------------------------------------------------------------------------------"
	write-host @InfoColor "                             Copying symbol DLL's ..."
	write-host @InfoColor "---------------------------------------------------------------------------------------------------------------------------------------------"
	$CopyResult = (copy-item (join-path $DllSourceParh "*") $SymbolShareCustomSymbols -force -recurse -passThru)
	write-host @InfoColor "--- $($CopyResult.count) files (DLL's) copied to Custom Symbols folder ---"
}
	
function CopyPartImages
{
	write-host @InfoColor "---------------------------------------------------------------------------------------------------------------------------------------------"
	write-host @InfoColor "                             Copying part images ..."
	write-host @InfoColor "---------------------------------------------------------------------------------------------------------------------------------------------"
	$CopyResult = (copy-item (join-path $PartImagesSourceParh "*") $SymbolSharePartImages -force -recurse -passThru)
	write-host @InfoColor "--- $($CopyResult.count) part images copied to PartImages folder ---"
}

function WaitForProcess
{
	param
	(
		[Parameter(mandatory=$true)]
		$proc,
		[Parameter(mandatory=$true)]
		[string]$Activity,
		[string]$Status = "Waiting for program exit ..."
	);
	$ii = 1000
	$p = 1
	$step = 1
	do
	{
		 Write-Progress -Activity $Activity -Status $Status -PercentComplete $p
		 $p += $step
		 if ($p -gt 100)
		 {
			$p = 99
			$step = -1
		 }
		 if ($p -lt 1)
		 {
			$p = 1
			$step = 1
		 }
		 Start-Sleep -Milliseconds 500
	} while ( !$proc.HasExited )
}

function StartS3dPrjMngmntAndWait
{
	write-host "Starting  S3D Project Management ..."
	$procPrjMngmnt = start-process $S3DHostExe -ArgumentList "/InitialXMLFile=""$PrjEnvXml"" TempSession" -PassThru
	
	write-host @WarningColor "********************************************************************************************************"
	write-host ""
	write-host @WarningColor "     Instructions"
	write-host @WarningColor "     ============"
	write-host ""
	write-host @WarningColor "         Steps to perform manually in S3D Project Management"
	write-host ""
	write-host ""
	write-host @WarningColor "         1) Select the catalog where the ""Update Custom Symbol configuration ..."" command should be executed"
	write-host @WarningColor "         2) Open Tools menu and click on ""Update Custom Symbol configuration ..."" and follow the instructions"
	write-host @WarningColor "         3) After the command finished, Close the S3D Project Management and this installation will continue!"
	write-host ""
	write-host @WarningColor "********************************************************************************************************"
	write-host ""
	
	
	WaitForProcess $procPrjMngmnt "Smart3D Project Management" "Waiting until program is closed ..."
}

function RunBulkload
{
	param (
		[string]$BulkloadMode,
		[string]$BulkloadFile,
		[string]$CodeListFile,
		[string]$LogFile
	);
	
	if ($BulkloadMode -eq "AMD")
	{
		# write-host "setting values for AMD mode"
		$a_val = 0
		$u_val = 1
	} 
	elseif ($BulkloadMode -eq "APPEND")
	{
		# write-host "setting values for APPEND mode"
		$a_val = 1
		$u_val = 0
	} else 
	{
		throw "******** Bulkload mode not AMD or APPEND - abort! *********************"
	}

	if ([string]::IsNullOrEmpty($BulkloadFile)) 
	{
		throw "RunBulkload: Bulkload file ""$BulkloadFile"" not specified - Abort!"
	}
	elseif (!(Test-Path -Path $BulkloadFile -PathType Leaf))
	{
		throw "RunBulkload: Bulkload file ""$BulkloadFile"" does not exist - Abort!"
	}
	if (!([string]::IsNullOrEmpty($CodeListFile)))
	{
		if (!(Test-Path -Path $CodeListFile -PathType Leaf))
		{
			throw "RunBulkload: Bulkload codelist file ""$CodeListFile"" does not exist - Abort!"
		}
	}
	
	if (!([string]::IsNullOrEmpty($CodeListFile))) 
	{
		# Call bulkload utility with code list file
		$BulkLoadArgs ="-i ""$BulkloadFile"" -ci ""$CodeListFile"" -l ""$LogFile"" -ds $DBserver -dn $DBcatalog -ss $DBserverSchema -sn $DBcatalogSchema -sp ""$SymbolShare"" -a $a_val -u $u_val -r 0 -f 1"
	}
	else
	{
		# Call bulkload utility WITH OUT code list file
		$BulkLoadArgs ="-i ""$BulkloadFile"" -l ""$LogFile"" -ds $DBserver -dn $DBcatalog -ss $DBserverSchema -sn $DBcatalogSchema -sp ""$SymbolShare"" -a $a_val -u $u_val -r 0 -f 1"
	}
	
	if (!([string]::IsNullOrEmpty($CodeListFile))) 
	{
		write-host @InfoColor "Running Bulkload of $([System.IO.Path]::GetFileName($BulkloadFile)) and CodeList $([System.IO.Path]::GetFileName($CodeListFile))"
	}
	else
	{
		write-host @InfoColor "Running Bulkload of $([System.IO.Path]::GetFileName($BulkloadFile)) with out code list"
	}
	
	$procBulkload = start-process "$BulkloadExe" -windowstyle Hidden  -ArgumentList "$($BulkLoadArgs)" -PassThru
	if ($ViewBulkloadLogs)
	{
		write-host "Opening new window to show bulkload log file content changes ..."
		start-process powershell.exe -argumentlist  "-command `$host.ui.RawUI.WindowTitle = '$([System.IO.Path]::GetFileName($logfile))'; While (!(Test-Path ""'$logfile'"")) {write-host '*** Waiting for log file creation ... ***'; Start-Sleep -Seconds 1}; get-content -Path ""'$logfile'"" -wait -tail 30"
	}
	WaitForProcess $procBulkload "Bulkloading $([System.IO.Path]::GetFileName($BulkloadFile))" "Waiting until bulkload.exe is finished ..."

	$SearchResult = Select-String -Path $LogFile -Pattern "Bulkload is successful with out any errors/warnings."
	if ($SearchResult -eq $null)
	{
		try
		{
			start-process notepad.exe -Argumentlist """$LogFile"""
		}
		catch
		{
		}
		throw "RunBulkload: Bulkload of ""$BulkloadFile"" failed: see log file: ""$LogFile"""
	}
}

function BulkloadAllLisegaFiles
{
	$xlsRootPath = [IO.Path]::Combine($PackageSource, "Excel Workbooks")
	# 4. Run bulkload in Append mode to add the LISEGA catalog to the catalog database 
	write-host @InfoColor "Start bulkload 1 of 3 ..."
	$Mode1 = "APPEND"
	$xlsPath1 =[IO.Path]::Combine($xlsRootPath, "01 - Catalog")
	$xls11 = Join-Path $xlsPath1 "LISEGA.S3D.Catalog.xlsx"
	$xls12 = Join-Path $xlsPath1 "LISEGA.S3D.Catalog_CodeList.xlsx" 
	$log1  = Join-Path $xlsPath1 "LISEGA.S3D.Catalog.log"
	#echo. call SubBulkload 1
	RunBulkload -BulkloadMode $Mode1 -BulkloadFile $xls11 -CodeListFile $xls12 -LogFile $log1
	
	# 5. Run Bulkload in Append mode to add plug-in attributes to the LISEGA catalog classes 
	write-host @InfoColor "Start bulkload 2 of 3 ..."
	$Mode2 = "APPEND"
	$xlsPath2 = Join-Path $xlsRootPath "02 - catalog plugin attributes" 
	$xls21 = Join-Path $xlsPath2 "LISEGA.S3D.Catalog.PluginAttributes.xlsx"
	$xls22 = Join-Path $xlsPath2 "LISEGA.S3D.Catalog.PluginAttributes_CodeList.xlsx" 
	$log2  = Join-Path $xlsPath2 "LISEGA.S3D.Catalog.PluginAttributes.log"
	#echo. call SubBulkload 1
	RunBulkload -BulkloadMode $Mode2 -BulkloadFile $xls21 -CodeListFile $xls22 -LogFile $log2

	# 6. Run Bulkload in AMD mode to activate the LISEGA H&S plug-in
	write-host @InfoColor "Start bulkload 3 of 3 ..."
	$Mode3 = "AMD"
	$xlsPath3 = Join-Path $xlsRootPath "03 - Plugin" 
	$xls31 = Join-Path $xlsPath3 "LISEGA_HS_Plugin_Activate_AMD.xlsx"
	$xls32 = $null
	$log3  = Join-Path $xlsPath3 "LISEGA_HS_Plugin_Activate_AMD.log"
	RunBulkload -BulkloadMode $Mode3 -BulkloadFile $xls31 -CodeListFile $xls32 -LogFile $log3
	
}

function LogOptHint
{
	if ($DBserverSchemaWasEmpty -eq $true) { " -- Hint: Value copied from DBserver, because parameter was not specified in command line!" }
}

function LogVariables
{
	write-host ""
	write-host ""
	write-host "PowerShell Parameter and variable values:"
	write-host "-----------------------------------------------------------------------------------------------"
	write-host "   PackageSource              = ""$PackageSource"""
	write-host "   DllSourceParh              = ""$DllSourceParh"""
	write-host "   PartImagesSourceParh       = ""$PartImagesSourceParh"""

	write-host "   S3DinstDir                 = ""$S3DinstDir"""
	write-host "   SymbolShare                = ""$SymbolShare"""
	write-host "   DBserver                   = ""$DBserver"""
	write-host "   DBcatalog                  = ""$DBcatalog"""
	write-host "   DBcatalogSchema            = ""$DBcatalogSchema"""
	write-host "   DBserverSchema             = ""$DBserverSchema"" $(LogOptHint)"
	write-host "   ViewBulkloadLogs           = ""$ViewBulkloadLogs"""
	write-host "   SkipFileCopy               = ""$SkipFileCopy"""
	write-host "   SkipPrjMngmnt              = ""$SkipPrjMngmnt"""
	write-host "   SkipBulkloads              = ""$SkipBulkloads"""
	write-host "   S3dInstDirCoreRuntime      = ""$S3dInstDirCoreRuntime"""
	write-host "   S3dInstDirCoreContainerBin = ""$S3dInstDirCoreContainerBin"""
	write-host "   S3dHostExe	              = ""$S3dHostExe""" 
	write-host "   PrjEnvXml                  = ""$PrjEnvXml"""
	write-host "   BulkloadExe                = ""$BulkloadExe"""
	
	
}

#This line executes the program
& $Main